<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
</head>
<body>
    <div class="sidebar">
        {% include "dynamic_preferences/sections.html" %}
    </div>
    <div class="main_content">
        {% block content %}{% endblock %}
    </div>
</body>
</html>